"""Main starter module.

The DEBUG constant can be safely assumed set by the time this module is
imported.

"""

from __future__ import division


import random, math
import datetime

import logging
log = logging.getLogger('main')


import yaml 

import cocos
import cocos.scene
from cocos.scenes.transitions import *
from cocos.scene import Scene
from cocos.sprite import Sprite
from cocos.director import director

import pyglet
import data
import config
from common import *
from constants import *
import game_audio

from bg_layer import BGLayer
from menu_main import MainMenu
try:
    import numpy
    
    print "Imported numpy!"
except ImportError:
    log.error( "Import numpy failed.")
try:
    from PIL import Image
    from PIL import TgaImagePlugin
    from PIL import PngImagePlugin
    
    print "Imported PIL!"
except ImportError:
    log.error( "Import PIL failed.")
  
def run():
    
    director.init(caption=CAPTION, width=800, height=600, resizable=False )

 
    mainmenu = Scene(BGLayer("menu"), MainMenu())
    intro = Scene(BGLayer("fruti_fly_title"))
    game_audio.play_song('music_intro.ogg')
    

    director.window.set_fullscreen(config.fullscreen)

    image = pyglet.image.load('data/images/mouse.png')
    cursor = pyglet.window.ImageMouseCursor(image, 16, 16)
    director.window.set_mouse_cursor(cursor)
    
    if config.playerseed == "": # first time this person has run the game
            print "First time run generating seed"
            config.playerseed = str(random.uniform(0, 99999999))+":"+datetime.datetime.now().strftime("%Y%m%d-%H%M%S")
            config.save_option("playerseed", config.playerseed)
            director.run( FadeTransition( intro, 5, mainmenu) )
    
    else:   
        director.run( FadeTransition( mainmenu, 5, intro) )
    

def main():
    log.info('Starting main...')  
    
    if config.profile:
        import os, profile
        timestamp = datetime.datetime.now().strftime("%Y%m%d-%H%M%S")
        filename = os.path.join(SCRIPT_DIR, "profile-%s.log" % timestamp)
        profile.runctx("run()", globals(), None, filename)
        import pstats # http://docs.python.org/library/profile.html
        p = pstats.Stats(filename)
        stats = p.sort_stats('time').print_stats(10)#print_callers(10) 
        log.debug(stats)
    else:
        run()
    
    log.info('... DONE')  
    
